/****************************************************************************
 *
 * Proiectare cu Microprocesoare, 2018
 * Author: Iulian Matesica <iulian.matesica@gmail.com>
 *
 ****************************************************************************/
#define F_CPU 16000000
#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>
#include <inttypes.h>

#include "game.h"
#include "vector.h"
#include "ili9341.h"
#include "pff.h"


/****************************************************************************
 * Preprocessor Definitions
 ****************************************************************************/

#define GAME_WINDOW_WIDTH  ILI9341_TFTWIDTH
#define GAME_WINDOW_HEIGHT ILI9341_TFTHEIGHT

#define NUMBER_LENGTH  40
#define BRICK_WEAK   1
#define BRICK_HARD   2

#define NUMBER_SPACE 7
#define NUMBER_STARTX 30
#define NUMBER_STARTY 30
 
#define BRICKS_VMAX   5
#define BRICKS_HMAX   10

FATFS fs;					// sistemul de fisiere


/****************************************************************************
 * Private Types
 ****************************************************************************/

struct number
{
  /* Number's position. */

  struct vector2i_s pos;

  /* Number's value. */

  int val;
};

enum game_status_e
{
    GAME_RUNNING,
    GAME_LOST,
    GAME_WON,
};

struct game_t
{
    /* Q: What's red and it's not good for you teeth?
     * A: A RED BRICK!
     */

    /* Current game status. */

    uint8_t     status;

    /* Bricks. */

    struct number     numbers[4][4];
};

/****************************************************************************
 * Private Data
 ****************************************************************************/

static struct game_t game;
static int score;

/****************************************************************************
 * Private Functions Prototypes
 ****************************************************************************/

static void GAME_draw_bricks(void);
static void update_score(void);

/****************************************************************************
 * Public Functions
 ****************************************************************************/


void start_window(void)
{
	ili9341_fillrect(0, 0, ILI9341_TFTWIDTH, ILI9341_TFTHEIGHT, BLACK);

	ili9341_setRotation(3);
	ili9341_setcursor(30,60);
	_delay_ms(2);
	ili9341_settextcolour(WHITE,BLACK);
	_delay_ms(2);
	ili9341_settextsize(3);
	_delay_ms(2);
	printf("Apasati butonul");
	ili9341_setcursor(40,100);
	_delay_ms(2);
	printf("joystick-ului");
	ili9341_setcursor(30,140);
	_delay_ms(2);
	printf("pentru a incepe!");
}

int verify_game_won(void) {
	int i, j;
	for (i = 0; i < 4; i++) {
		for (j = 0; j < 4; j++) {
			if (game.numbers[i][j].val == 2048) {
				game.status = GAME_WON;
				return 1;
			}
		}
	}
	return 0;
}

int verify_game_lost(void) {
	int i, j;
	for (i = 0; i < 4; i++) {
		for (j = 0; j < 4; j++) {
			if (game.numbers[i][j].val == 0) 
				return 0;
			if (j < 3 && game.numbers[i][j+1].val == game.numbers[i][j].val)
				return 0;
			if (i < 3 && game.numbers[i+1][j].val == game.numbers[i][j].val)
				return 0;
			if (i > 0 && game.numbers[i-1][j].val == game.numbers[i][j].val)
				return 0;
			if (j > 0 && game.numbers[i][j-1].val == game.numbers[i][j].val)
				return 0;
		}
	}
	game.status == GAME_LOST;
	return 1;
}

void atribuire(int b[4][4])
{
	int i,j;
	for(i = 0; i < 4; i++)					/*functia creaza o copie a unei matrice*/
		for(j = 0; j < 4; j++)
			b[i][j] = game.numbers[i][j].val;
}

int comparatie(int b[4][4])
{
	int i,j;
	for(i = 0; i < 4; i++)					/*functia compara elementele a doua matrici*/
		for(j = 0; j < 4; j++)				/*trimise ca parametri*/
			if(game.numbers[i][j].val != b[i][j])
				return 1;
	return 0;
}

void GAME_init(void)
{

    for (int i = 0; i < 4; i++)
    {
        for (int j = 0; j < 4; j++)
        {
            struct number *n = &game.numbers[i][j];

            /* Brick's position. */

            n->pos.x = NUMBER_STARTX + j * (NUMBER_SPACE + NUMBER_LENGTH);
            n->pos.y = NUMBER_STARTY + i * (NUMBER_SPACE + NUMBER_LENGTH);
			n->val = 0;
        }
    }
	score = 0;

	game.numbers[rand() % 4][rand() % 4].val = (rand() % 10) ? 2 : 4;;
	game.numbers[rand() % 4][rand() % 4].val = (rand() % 10) ? 2 : 4;;

    /* Empty screen. */

    ili9341_fillrect(0, 0, ILI9341_TFTWIDTH, ILI9341_TFTHEIGHT, BLACK);
    ili9341_setRotation(3);
    ili9341_setcursor(240,50);
    _delay_ms(2);
    ili9341_settextcolour(WHITE,BLACK);
    _delay_ms(2);
    ili9341_settextsize(2);
    _delay_ms(2);
    printf("Scor: ");
	ili9341_setcursor(240,70);
	_delay_ms(2);
	printf("%d", score);

	/* Draw them. */

    GAME_draw_bricks();

    /* Start the game! */

    game.status = GAME_RUNNING;
}

void stanga(int i, int k)
{
	int j, ok = 0;
	do
	{for(j = k; j < 3; j++)
		if(game.numbers[i][j].val == 0)			/*functia deplaseaza elementele matricei*/
		{game.numbers[i][j].val = game.numbers[i][j+1].val;		/*la stanga daca este loc*/
		game.numbers[i][j+1].val = 0;}
	ok++;}
	while(ok < 3);
}

void dreapta(int i, int k)
{
	int j, ok = 0;
	do
	{for(j = k; j >= 1; j--)
		if(game.numbers[i][j].val == 0)			/*functia deplaseaza elementele matricei*/
		{game.numbers[i][j].val = game.numbers[i][j-1].val;		/*la dreapta daca este loc*/
		game.numbers[i][j-1].val = 0;}
	ok++;}
	while(ok < 3);
}

void sus(int k, int j)
{
	int i, ok = 0;
	do
	{for(i = k; i < 3; i++)
		if(game.numbers[i][j].val == 0)			/*functia deplaseaza elementele matricei*/
		{game.numbers[i][j].val = game.numbers[i+1][j].val;		/*in sus daca este loc*/
		game.numbers[i+1][j].val = 0;}
	ok++;}
	while(ok < 3);
}

void jos(int k, int j)
{
	int i, ok = 0;
	do
	{for(i = k; i >= 1; i--)
		if(game.numbers[i][j].val == 0)			/*functia deplaseaza elementele matricei*/
		{game.numbers[i][j].val = game.numbers[i-1][j].val;		/*in jos daca este jos*/
		game.numbers[i-1][j].val = 0;}
	ok++;}
	while(ok < 3);
}

void aparitie(void)
{
	int random_number, i, j;
	random_number = 0;
	do
		{i = rand() % 4;
		j = rand() % 4;}				/*functia genereaza aparitia aleatoare*/
	while(game.numbers[i][j].val != 0);					/*a unui numar din multimea {2,4}*/
	while (random_number == 0)
		random_number = 2 + (2*rand()) % 4;
	game.numbers[i][j].val = random_number;
}

void GAME_loop(void)
{
	int b[4][4];
    /* Check if the player lost or won. */
	
	int x,y;
	int i, j;
	x = ADC_get(3);
	y = ADC_get(2);

	DDRD |= (1 << PD4);

	if (y == 1023) {					//dreapta
		atribuire(b);
		for(i = 0; i < 4; i++)
			for(j = 3; j >= 1; j--)
			{
				dreapta(i,j);
				if(game.numbers[i][j].val == game.numbers[i][j-1].val)
				{
					game.numbers[i][j].val = game.numbers[i][j].val + game.numbers[i][j-1].val;
					game.numbers[i][j-1].val = 0;
					dreapta(i,j);
					score = score + game.numbers[i][j].val;
				}
			}
		if(comparatie(b) == 1) {
			aparitie();
			GAME_draw_bricks();
			update_score();
		}
		for (int i = 0; i < 50; ++i) {
			PIND = (1 << PD4);

			_delay_ms(1);
		}
	} else if (x == 1023) {				//sus
		atribuire(b);
		for(i = 0; i < 3; i++)
			for(j = 0; j < 4; j++)
			{
				sus(i,j);
				if(game.numbers[i][j].val == game.numbers[i+1][j].val)
				{
					game.numbers[i][j].val = game.numbers[i][j].val + game.numbers[i+1][j].val;
					game.numbers[i+1][j].val = 0;
					sus(i,j);
					score = score + game.numbers[i][j].val;
				}
			}
		if(comparatie(b) == 1) {
			aparitie();
			GAME_draw_bricks();
			update_score();
		}
		for (int i = 0; i < 50; ++i) {
			PIND = (1 << PD4);

			_delay_ms(1);
		}
	} else if (y == 0) {				//stanga
		atribuire(b);
		for(i = 0; i < 4; i++)
			for(j = 0; j < 3; j++)
			{
				stanga(i,j);
				if(game.numbers[i][j].val == game.numbers[i][j+1].val)
				{
					game.numbers[i][j].val = game.numbers[i][j].val + game.numbers[i][j+1].val;
					game.numbers[i][j+1].val = 0;
					stanga(i,j);
					score = score + game.numbers[i][j].val;
				}
			}
		if(comparatie(b) == 1) {
			aparitie();
			GAME_draw_bricks();
			update_score();
		}
		for (int i = 0; i < 50; ++i) {
			PIND = (1 << PD4);

			_delay_ms(1);
		}
	} else if (x == 0) {				//jos
		atribuire(b);
		for(i = 3; i >= 1; i--)
			for(j = 0; j < 4; j++)
			{
				jos(i,j);
				if(game.numbers[i][j].val == game.numbers[i-1][j].val)
				{
					game.numbers[i][j].val = game.numbers[i][j].val + game.numbers[i-1][j].val;
					game.numbers[i-1][j].val = 0;
					jos(i,j);
					score = score + game.numbers[i][j].val;
				}
			}
		if(comparatie(b) == 1) {
			aparitie();
			GAME_draw_bricks();
			update_score();
		}
		for (int i = 0; i < 50; ++i) {
			PIND = (1 << PD4);

			_delay_ms(1);
		}
	}
	PORTD &= ~(1 << PD4);
}

/****************************************************************************
 * Private Functions
 ****************************************************************************/


static void GAME_draw_bricks(void)
{
    for (int i = 0; i < 4; i++)
    {
        for (int j = 0; j < 4; j++)
        {
            struct number *n = &game.numbers[i][j];
			ili9341_setcursor(n->pos.x + 10, n->pos.y + 10);
			_delay_ms(2);
			ili9341_settextsize(2);
			_delay_ms(2);

			if (n->val == 0) {
				ili9341_fillrect(n->pos.x, n->pos.y,
								 NUMBER_LENGTH,
								 NUMBER_LENGTH,
								 RED);
			} else if (n->val == 2) {
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								YELLOW);
				ili9341_settextcolour(WHITE,YELLOW);
				_delay_ms(2);
				printf("%d", n->val);
			} else if (n->val == 4) {
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								GREEN);
				ili9341_settextcolour(WHITE,GREEN);
				_delay_ms(2);
				printf("%d", n->val);
			} else if (n->val == 8) {
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								BLUE);
				ili9341_settextcolour(WHITE,BLUE);
				_delay_ms(2);
				printf("%d", n->val);
			} else if (n->val == 16) {
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								MAGENTA);
				ili9341_settextcolour(WHITE,MAGENTA);
				_delay_ms(2);
				printf("%d", n->val);
			} else if (n->val == 32) {
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								ORANGE);
				ili9341_settextcolour(WHITE,ORANGE);
				_delay_ms(2);
				printf("%d", n->val);
			} else if (n->val == 64) {
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								NAVY);
				ili9341_settextcolour(WHITE,NAVY);
				_delay_ms(2);
				printf("%d", n->val);
			} else if (n->val == 128) {
				ili9341_setcursor(n->pos.x, n->pos.y + 10);
				_delay_ms(2);
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								DARKCYAN);
				ili9341_settextcolour(WHITE,DARKCYAN);
				_delay_ms(2);
				printf("%d", n->val);
			} else if (n->val == 256) {
				ili9341_setcursor(n->pos.x, n->pos.y + 10);
				_delay_ms(2);
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								PURPLE);
				ili9341_settextcolour(WHITE,PURPLE);
				_delay_ms(2);
				printf("%d", n->val);
			} else if (n->val == 512) {
				ili9341_setcursor(n->pos.x, n->pos.y + 10);
				_delay_ms(2);
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								OLIVE);
				ili9341_settextcolour(WHITE,OLIVE);
				_delay_ms(2);
				printf("%d", n->val);
			} else if (n->val == 1024) {
				ili9341_settextsize(1);
				_delay_ms(2);
				ili9341_setcursor(n->pos.x, n->pos.y + 10);
				_delay_ms(2);
				ili9341_fillrect(n->pos.x, n->pos.y,
								NUMBER_LENGTH,
								NUMBER_LENGTH,
								DARKGREEN);
				ili9341_settextcolour(WHITE,DARKGREEN);
				_delay_ms(2);
				printf("%d", n->val);
			}
		} 
    }
}

static void update_score(void) {
	 ili9341_setcursor(240,70);
	 _delay_ms(2);
	 ili9341_settextcolour(WHITE,BLACK);
	 _delay_ms(2);
	 ili9341_settextsize(2);
	 printf("%d", score);
}
